########################################### ESTIMATION FUNCTIONS ###########################################

chunk2 <- function(x,n) split(x, cut(seq_along(x), n, labels = FALSE)) # split a time serie into n chunks

bridge.detrend <- function(x){
  N <- length(x)
  bridge <- seq(x[1], x[N], length = N)
  x <- x - bridge
}

poly.detrend <- function(x){
  N <- length(x)
  t <- 1:N
  reg <- lm(x ~ t)
  poly_detrend <- x - (reg$coefficients[2]*t + reg$coefficients[1])
}

library(fractal)

# SWV :
SWV <- function(x,scale.min=4,scale.max=trunc(length(x)/2),scale.ratio=2, detrend,weighted=F){
  N <- length(x)
  scales <- logScale(scale.min, scale.max, scale.ratio = scale.ratio, coerce = trunc) # define the different scales
  SD <- vector()
  k <- 1
  for (tau in scales){ # for each possible size of window
    x_detrended <- list()
    SD_i <- vector()
    windows <- chunk2(x,N/tau) # split the time series into windows of length tau
    for (window in seq(windows)){ # within each window, detrend and calculate the standard deviation
      if (detrend=="bridge")  x_detrended[[window]] <- bridge.detrend(windows[[window]])
      else if (detrend=="poly")  x_detrended[[window]] <- poly.detrend(windows[[window]])
      else if (detrend=="none")  x_detrended[[window]] <- windows[[window]]
      else stop ("Not an implemented method, try \"bridge\" or \"poly\"")
      SD_i[window] <- sd(x_detrended[[window]])
    }
    SD[k] <- mean(SD_i)
    k <- k+1
  }
  res.SD <<- SD
  res.scales <<- scales
  ifelse(weighted==F, w <- rep(1,length(log(scales))), w <- 1/seq(along=log(scales)))
  return(unname(lm(log(SD)~1+log(scales),weights=w)$coefficients[2]))
}

# HAV : only works for fGn (= diff(fBm))
HAV <- function(x,scale.min=4,scale.max=trunc(length(x)/2),scale.ratio=2){
  N <- length(x)
  x_bar <- mean(x)
  scales <- logScale(scale.min, scale.max, scale.ratio = scale.ratio, coerce = trunc)
  absolute <- vector()
  k <- 1
  for (m in scales){ # m is the windows size (MacIntosh 2013)
    absolute_i <- vector()
    windows <- chunk2(x,N/m)
    for (window in seq(windows)){
      absolute_i[window] <- abs(mean(windows[[window]])-x_bar)
    }
    absolute[k] <- (m/N)*sum(absolute_i) # MacIntosh 2013
    k <- k+1
  }
  inf <- which(is.infinite(log(absolute)) | is.infinite(log(scales)))
  if (length(inf)==0){
  }
  else{
    absolute <- absolute[-inf]
    scales <- scales[-inf]
  }
  return(unname(lm(log(absolute)~1+log(scales))$coefficients[2])+1)
}

lowPSD <- function(x){
  n <- length(x)
  spec <- spectrum(x, detrend=FALSE, demean=TRUE, taper=0)
  nr <- (n/2) * (1/8)
  specfreq <- spec$freq[1:nr]
  specspec <- spec$spec[1:nr]
  logfreq <- log(specfreq)
  logspec <- log(specspec)
  lmb <- lm(logspec ~ logfreq)
  b <- coef(lmb)
  beta=-b[2]
  plot (logfreq, logspec, type="l")
  abline(lmb)
  beta
}

########################################### IMPORT TIME SERIES ###########################################
library(stringr)

files <- list.files(path = "C:/Users/Stage/Desktop/Stage_M2/donnees/Fichiers bruts et binaires/Groupes",full.names=TRUE,recursive=TRUE)
files <- tolower(files)
files <- files[grepl("binary",files)]

time_series <- list()

for (i in 1:length(files)){
  time_series[[i]]<- read.csv(files[i], header = F)
  print(i)
}

names(time_series) <- tolower(gsub(tolower("C:/Users/Stage/Desktop/Stage M2/donnees/Fichiers bruts et binaires/Groupes/"),"",files))
names(time_series) <- gsub("-binary.csv","",names(time_series))

time_series <- lapply(time_series, function(x) unlist(x))

PS_idx <- which(str_detect(names(time_series),"petite section")) # 3 years old
MS_idx <- which(str_detect(names(time_series),"moyenne section")) # 4 years old
GS_idx <- which(str_detect(names(time_series),"grande section")) # 5 years old
CE1_idx <- which(str_detect(names(time_series),"ce1"))# 7 years old
CE2_idx <- which(str_detect(names(time_series),"ce2")) # 8 years old
CM1_idx <- which(str_detect(names(time_series),"cm1")) # 9 years old
CM2_idx <- which(str_detect(names(time_series),"cm2")) # 10 years old
NOV_idx <- which(str_detect(names(time_series),"novice")) # naive adults
EXP_idx <- which(str_detect(names(time_series),"expert")) # experts adults


age <- vector()
age[PS_idx] <- "3" ; age[MS_idx] <- "4" ; age[GS_idx] <- "5" ; age[CE1_idx] <- "7" ; age[CE2_idx] <- "8"
age[CM1_idx] <- "9" ; age[CM2_idx] <- "10" ; age[NOV_idx] <- "NAIVE" ; age[EXP_idx] <- "EXPERT"

age <- ordered(age, levels = c("3","4","5","7","8","9","10","NAIVE","EXPERT"))

free_idx <- which(str_detect(names(time_series),"libre"))
self_portrait_idx <- which(str_detect(names(time_series),"consigne"))

condition <- vector()
condition[libre_idx] <- "FREE"
condition[consigne_idx] <- "SELF-PORTRAIT"

df <- as.data.frame(cbind(1:length(time_series),age,condition))
df$names <- names(time_series)
names(df) <- c("index","age","condition","names")

levels(groupe) = c("3","4","5","7","8","9","10","NAIVE","EXPERT")

names(time_series) <- unlist(strsplit(names(time_series),"/",fixed=TRUE))[(1:370)*11]

names(time_series) <- substr(names(time_series),1,7)
names(time_series)[which(duplicated(df[,c(3,4)]))]
df$names <- names(time_series)

lengths <- unlist(lapply(time_series,function(x) length(x)))

df$age <- as.character(age)

df <- as.data.frame(cbind(names(time_series),condition,age,lengths))
df$age <- as.character(age)
names(df)[1] = "names"

time_series[[316]] = NULL
df = df[-316,]
rm(list=setdiff(ls(),c("df","time_series")))

########################################### HURST ESTIMATIONS ########################################### 

library(fractal)

df$DFA = NA
for (i in 1:dim(df)[1]){
  df$DFA[i] = DFA(DFA(time_series[[i]],sum.order=1,detrend="bridge")%%1)
}

gt_50000 = which(df$lengths>50000)

# create the dataframe for drawings with length > 50 seconds :
df_50000 = data.frame(names=as.character(),condition=as.character(),age=as.character(),
                      DFA_50000=as.numeric(),HAV_50000=as.numeric(),SWV_50000=as.numeric(),hSpec_50000=as.numeric(),lowPSD_50000=as.numeric(),
                      stringsAsFactors = FALSE)


# calcul des estimations de H pour les dessins >50 000 points :
j=1
for (i in gt_50000_rajout){
  df_50000[j,] = c(as.character(df$names[i]),as.character(df$condition[i]),as.character(df$age[i]),
                   DFA(time_series[[i]][1:50000],sum.order=1,detrend="bridge")%%1,
                   HAV(diff(time_series[[i]][1:50000])),
                   SWV(time_series[[i]][1:50000],detrend="bridge"),
                   hurstSpec(time_series[[i]][1:50000])%%1,
                   (lowPSD(time_series[[i]][1:50000])-1)/2)
  print(j)
  j=j+1
}

for (i in 4:8){
  df_50000[,i] = as.numeric(df_50000[,i])
}

########################################### PCA ########################################### 

library(Rcmdr)
library(RcmdrPlugin.FactoMineR)

# PCA on the first 50 seconds :
df_50000.PCA<-df_50000[, c("DFA_50000", "HAV_50000", "SWV_50000", 
                           "lowPSD_50000")]
res<-PCA(df_50000.PCA , scale.unit=TRUE, ncp=5, graph = FALSE)
df_50000$Hurst_axis = as.vector(res$ind$coord[,1])

# loadings :

sweep(res$var$coord,2,sqrt(res$eig[1:ncol(res$var$coord),1]),FUN="/")
########################################### IMPORT ALREADY CALCULATED HURST INDICES ###########################################

df_50000 = readxl::read_xlsx("data_hurst.xlsx")
########################################### MIXED MODEL ########################################### 

library(nlme)
library(lsmeans)
library(MuMIn)

mod_complete <- lme(fixed = Hurst_axis ~ age + condition + sex + age:condition + condition:sex,
                  random = ~ 1|names,
                  weights = varIdent(form =~ 1 | age),
                  method = "ML",
                  data = df_50000)

# model selection :

# dredge
dredge_mod = dredge(mod_complete)

dredge_mod_list = get.models(dredge_mod,1:4)
dredge_mod

best_model <- lme(fixed = Hurst_axis ~ age,
                  random = ~ 1|names,
                  method = "REML",
                  data = df_50000)

# diagnostics :
plot(best_model) # heteroscedasticity
boxplot(residuals(best_model) ~ age, data=df_50000) # different variation between groups

# normalite des residus :
qqnorm(residuals(best_model))
qqline(residuals(best_model))

# adding a covariance structure :
mod_final <- lme(fixed = Hurst_axis ~ age,
                 random = ~ 1|names,
                 weights = varIdent(form =~ 1 | age),
                 method = "REML",
                 data = df_50000)

lsmeans(mod_final, pairwise ~ age)
